/*
 * Copyright (C) 2010  Michael Morin
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the  v. 3.0  or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the Academic Free License ("AFL")
 * v. 3.0  for more details.
 *
 * You should have received a copy of the Academic Free License ("AFL") v. 3.0
 * along with this program.
 *
 */

/*
 * From the author:
 * This software uses the following third party libraries:
 *  - Choco Solver 2.1.3 (http://www.emn.fr/z-info/choco-solver)
 *  - Jung 2.0.1 (http://jung.sourceforge.net)
 * and is related to the following publication:
 *  M. Morin, A.P. Papillon, F. Laviolette, I. Abi-Zeid, and C.G. Quimper, 
 *  "Constraint Programming for Path Planning with Uncertainty: Solving the 
 *  Optimal Search Path problem," in Proceedings of the 18th Conference on 
 *  Principles and Practice of Constraint Programming, Qubec, Qc, Canada, 
 *  2012, pp. 988-1003.
 * 
 * Enhancements and comments are welcomed. Please let us know!  
 *
 * Best regards,
 *
 * Michael
 * http://www.michaelmorin.info
 */


/** 
 * @file MainExperiment.java
 * @brief The main class file.
 */

package ospCp2012;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;


/**
 * @class MainExperiment
 * @version 1.0
 * @author Michael Morin
 * @date Created on: 2011-04-07
 * @date Modified on: 2012-11-02
 *  
 * @brief  class MainExperiment
 *
 * This class is the main class. Its purpose is to run the CP 2012 experiment
 * involving the Total Detection heuristic.
 * The only available experiment type is the OSP (Optimal Search Path) problem
 * type.
 * <p>
 * The application uses a command line interface where the algorithm (solver) 
 * and the problem instance configuration are specified as input files.
 * The method 'displayHelp' explain the command line and the input file 
 * formats.
 */
public class MainExperiment {

    private static final int EXPERIMENT_TYPE_OSP = 3;
    
    private static final int CONST_BACKTRACK_LIMIT = 5000000;
	private static final int CONST_TIME_LIMIT_MS = 1200000;
	private static final int CONST_SCALE_UP_FACTOR = 10000;

	public static void main(String[] args) {
		System.out.println(">>>> CP 2012 OSP Experiment >>>>");
		
		int verbose_level = 1;
			// These verbose settings are incremental:
			// 0 => No screen output during the solving process.
			// 1 => Minimal output (e.g., real objective value, solving time)
			// 2 => Set Choco to verbose
			// 3 => Display the solution at the end
			// 4 => Choco display all the encountered solutions
		boolean generate_csv_output = true;
		int backtrack_limit = CONST_BACKTRACK_LIMIT;
		int time_limit_ms = CONST_TIME_LIMIT_MS;
		int scale_up_factor = CONST_SCALE_UP_FACTOR;
		String output_filename = "output.output";
		String choco_logging_output_filename = "";
		
		if(args.length != 2) {
			displayHelp();
			return;
		} else {
			output_filename = new String(args[1]);
						
			Properties input_properties = new Properties();
			FileInputStream input_file = null;
			try {
				input_file = new FileInputStream(args[0]);
				input_properties.load(input_file);
			} catch (FileNotFoundException e) {
			    System.err.println("FileNotFoundException: "
			                        + e.getMessage());
			    return;
			} catch (IOException e) {
			    System.err.println("Caught IOException: "
			                        + e.getMessage());
			    return;
			}
			
			//boolean is_copwin_experiment = false;
			int experiment_type = 0; 
			int environment_id = 0;
			int config_id = 0;
			
			// What is the type of the experiment.
			if(input_properties.getProperty("Experiment").equals("osp") == true) {
				experiment_type = EXPERIMENT_TYPE_OSP;
			} else {
				displayHelp();
				return;
			}
			
			// Get the environment number
			environment_id = Integer.parseInt(input_properties.getProperty("Instance", "0"));
			
			// Get the configuration id for this "ospv" or "osp" experiment.
			if(experiment_type == EXPERIMENT_TYPE_OSP) {
				config_id = Integer.parseInt(input_properties.getProperty("Configuration", "0"));
				// Get the backtrack limit.
				backtrack_limit = Integer.parseInt(input_properties.getProperty("Backtrack", String.valueOf(backtrack_limit)));
				time_limit_ms = Integer.parseInt(input_properties.getProperty("Time", String.valueOf(time_limit_ms)));
				scale_up_factor = Integer.parseInt(input_properties.getProperty("ScaleUpFactor", String.valueOf(scale_up_factor)));
				
				if((verbose_level < 0) || (verbose_level > 4)) {
					displayHelp();
					return;
				}
				if(input_properties.getProperty("ChocoLogging", "-").equals("-") == false) {
					choco_logging_output_filename = input_properties.getProperty("ChocoLogging", "-");
				} else {
					choco_logging_output_filename = "";
				}
			} 
			verbose_level = Integer.parseInt(input_properties.getProperty("Verbosity", String.valueOf(verbose_level)));
			choco_logging_output_filename = input_properties.getProperty("ChocoLogging", "-");
			
			System.out.println("Your choices:");
			System.out.println("   1. The type of experiment: " + input_properties.getProperty("Experiment"));
			System.out.println("   2. The problem instance id: " + input_properties.getProperty("Instance", "0"));
			if(experiment_type == EXPERIMENT_TYPE_OSP) {
				System.out.println("   3. The configuration id: "  + input_properties.getProperty("Configuration", "0"));
				System.out.println("   4. Backtrack limit: "  + input_properties.getProperty("Backtrack", String.valueOf(backtrack_limit)));
				System.out.println("   5. Time limit (ms): "  + input_properties.getProperty("Time", String.valueOf(time_limit_ms)));
				System.out.println("   6. Scale-up factor (int > 0): "  + input_properties.getProperty("ScaleUpFactor", String.valueOf(scale_up_factor)));
				System.out.println("   7. The verbosity level of Choco Logging (default = 1): " + input_properties.getProperty("Verbosity", String.valueOf(verbose_level)));
				System.out.println("   8. The name of the file used for Choco Logging: " + input_properties.getProperty("ChocoLogging", "-"));
			} 
			
			// Run the experiment			
			InstanceConfig instance = new InstanceConfig(environment_id);
			if(experiment_type == EXPERIMENT_TYPE_OSP) {
				OspProblem osp_prob = new OspProblem();
				OspConfig osp_conf = new OspConfig();
				osp_conf.setToConfigId(config_id, backtrack_limit, time_limit_ms, scale_up_factor);
				osp_prob.generateOsp(instance);
				osp_prob.solveUsingOspModel(osp_conf, generate_csv_output, output_filename, verbose_level, choco_logging_output_filename);		
			} else {
				displayHelp();
			}
		}
		System.out.println("<<<< CP 2012 OSP Experiment <<<<");
	

	}
	
	static void displayHelp() {
		System.out.println("You must specify the configuration file to load and the output file to write.");
		System.out.println("Example: java -jar ospv_copwin.jar config.input result.output");
		System.out.println();
		System.out.println("Each configuration file contains the following information:");
		System.out.println("   Experiment = The type of experiment: [osp]");
		System.out.println("   Instance = The problem instance id: [#]");
		System.out.println("   Configuration = The solver configuration id: [#]");
		System.out.println("   Backtrack = Backtrack limit (default = " + CONST_BACKTRACK_LIMIT +  "): [#]");
		System.out.println("   Time = Time limit in milliseconds (default = " + CONST_TIME_LIMIT_MS +  "): [#]");
		System.out.println("   ScaleUpFactor = The integer used to multiple float probabilities (default = " + CONST_SCALE_UP_FACTOR +  "): [#]");
		System.out.println("   Verbosity = The verbosity level of Choco Logging (default = 1): [0] => no output");
		System.out.println("      [1] => Minimal output (e.g., real objective value, solving time");
		System.out.println("      [2] => More output Ex. Set Choco to Verbose");
		System.out.println("      [3] => Show the solution at the end of the solving process");
		System.out.println("      [4] => When Choco is used, Choco shows all the solutions including their objective values in ChocoLogging");
		System.out.println("   ChocoLogging = The name of the file used for Choco Logging: [-] for none or [filename]");
			// 0 => No screen output during the solving process.
			// 1 => Minimal output (e.g., real objective value, solving time)
			// 2 => Set Choco to verbose
			// 3 => Display the solution
		System.out.println("");
	}

}

// End of file
