/*
 * Copyright (C) 2010  Michael Morin
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the  v. 3.0  or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the Academic Free License ("AFL")
 * v. 3.0  for more details.
 *
 * You should have received a copy of the Academic Free License ("AFL") v. 3.0
 * along with this program.
 *
 */

/*
 * From the author:
 * This software uses the following third party libraries:
 *  - Choco Solver 2.1.3 (http://www.emn.fr/z-info/choco-solver)
 *  - Jung 2.0.1 (http://jung.sourceforge.net)
 * and is related to the following publication:
 *  M. Morin, A.P. Papillon, F. Laviolette, I. Abi-Zeid, and C.G. Quimper, 
 *  "Constraint Programming for Path Planning with Uncertainty: Solving the 
 *  Optimal Search Path problem," in Proceedings of the 18th Conference on 
 *  Principles and Practice of Constraint Programming, Qubec, Qc, Canada, 
 *  2012, pp. 988-1003.
 * 
 * Enhancements and comments are welcomed. Please let us know!  
 *
 * Best regards,
 *
 * Michael
 * http://www.michaelmorin.info
 */

/** 
 * @file OspConfig.java
 * @brief The OspConfig class.
 */

package ospCp2012;

/**
 * @class OspConfig
 * @version 1.0
 * @author Michael Morin
 * @date Created on: 2011-04-10
 * @date Modified on: 2012-04-07
 *  
 * @brief  class OspConfig
 *
 * This class is a configuration generator based on the configuration 
 * identifier.
 * <p>
 * Each configuration identifier has nine digits. The first digit is kept for
 * further usage. The second and the third digits determine the CP model type,
 * i.e., the objective function used. The fourth and fifth digits determine
 * the branching strategy. The sixth and seventh digits are kept for further 
 * usage. The eighth and ninth digits determine if any supplementary
 * enhancement are performed on the model, e.g., the filtering of the domains
 * of the variables.
 * 
 */
public class OspConfig {
	protected static final int MODEL_SUM = 1;
	protected static final int MODEL_MAX = 2;
	
	protected static final int BRANCHING_STRATEGY_DEFAULT = 1;
	protected static final int BRANCHING_STRATEGY_STATIC = 2;
	protected static final int BRANCHING_STRATEGY_STATIC_FAS_DYNAMIC_VALUE_ITERATOR = 5;
	
	protected static final int SUPPLEMENTARY_CONFIG_NONE = 1;
	protected static final int SUPPLEMENTARY_CONFIG_PROBABILITIES_DOMAIN = 2;
	
	int mId; // Configuration id
	
	// Configuration options
	boolean mUsePocUpperBoundFilter;
	boolean mUsePosUpperBoundFilter;
	int mProbaScaleUpFactor;
	int mBranchingStrategy;
	
	// Solver specific configurations
	int mBacktrackLimit;
		// Total number of backtrack allowed to the solver.
		// < 0 => no backtrack limit
	int mTimeLimitMs;
		// Allowed time in milliseconds
		// < 0 => no time limit
	
	int mModel;

	
	OspConfig() {
		setToConfigId(0, 0, 0, 100);
	}
	
	/**
	 * Generate a solver configuration.
	 * 
	 * @param id the configuration identifier
	 * @param backtrackLimit the backtrack limit
	 * @param timeLimitMs the time limit in milliseconds
	 * @param scaleUpFactor the scale up factor for probabilities
	 */
	public void setToConfigId(int id, int backtrackLimit, int timeLimitMs, int scaleUpFactor) {
		mId = id;
		mBacktrackLimit = backtrackLimit;
		mTimeLimitMs = timeLimitMs;
		mProbaScaleUpFactor = scaleUpFactor;
		
		if(id == 0) {
			mUsePocUpperBoundFilter = false;
			mUsePosUpperBoundFilter = false;
			mBranchingStrategy = BRANCHING_STRATEGY_DEFAULT;
			mModel = MODEL_SUM;

			return;
		}
		
		// Eight and ninth digits = Supplementary configurations
		int tmp_id = mId;
		int mod_10 = 0;
		for(int i = 0; i < 2; ++i) {
			mod_10 += (tmp_id % 10) * (int) Math.pow(10.0,(double) i);
			tmp_id = (tmp_id - (tmp_id % 10)) / 10;
		}
		switch (mod_10) {
	    	case SUPPLEMENTARY_CONFIG_NONE:  
				mUsePocUpperBoundFilter = false;
				mUsePosUpperBoundFilter = false;
	    		break;
	    	case SUPPLEMENTARY_CONFIG_PROBABILITIES_DOMAIN:  
				mUsePocUpperBoundFilter = true;
				mUsePosUpperBoundFilter = true;
	    		break;
	    	default: 
	    		System.out.println("Warning. The instance id is invalid. The supplementary configuration digits are invalid.");
	    		break;
	    }
		
		// Sixth and seventh digits = Not used
		mod_10 = 0;
		for(int i = 0; i < 2; ++i) {
			mod_10 += (tmp_id % 10) * (int) Math.pow(10.0,(double) i);
			tmp_id = (tmp_id - (tmp_id % 10)) / 10;
		}
		
		// Fourth and fifth digits = Branching strategy number
		mod_10 = 0;
		for(int i = 0; i < 2; ++i) {
			mod_10 += (tmp_id % 10) * (int) Math.pow(10.0,(double) i);
			tmp_id = (tmp_id - (tmp_id % 10)) / 10;
		}
		switch (mod_10) {
	    	case BRANCHING_STRATEGY_DEFAULT:  
	    		mBranchingStrategy = BRANCHING_STRATEGY_DEFAULT;
	    		break;
	    	case BRANCHING_STRATEGY_STATIC:  
	    		mBranchingStrategy = BRANCHING_STRATEGY_STATIC;
	    		break;
	    	case BRANCHING_STRATEGY_STATIC_FAS_DYNAMIC_VALUE_ITERATOR:
	    		mBranchingStrategy = BRANCHING_STRATEGY_STATIC_FAS_DYNAMIC_VALUE_ITERATOR;
	    		break;
	    	default: 
	    		System.out.println("Warning. The instance id is invalid. The branching strategy digits are invalid.");
	    		break;
	    }
		
		// second and third digits = Model number
		mod_10 = 0;
		for(int i = 0; i < 2; ++i) {
			mod_10 += (tmp_id % 10) * (int) Math.pow(10.0,(double) i);
			tmp_id = (tmp_id - (tmp_id % 10)) / 10;
		}
		switch (mod_10) {
	    	case MODEL_SUM:  
	    		mModel = MODEL_SUM;
	    		break;
	    	case MODEL_MAX:  
	    		mModel = MODEL_MAX;
	    		break;
	    	default: 
	    		System.out.println("Warning. The instance id is invalid. The model number digits are invalid.");
	    		break;
	    }
		
		// The first digit is not used.
 	}
	
	public boolean UsePocUpperBoundFilter() {
		return mUsePocUpperBoundFilter;
	}
	
	public boolean UsePosUpperBoundFilter() {
		return mUsePosUpperBoundFilter;
	}
	
	public int getBacktrackLimit() {
		return mBacktrackLimit;
	}
	
	public int getBranchingStrategy() {
		return mBranchingStrategy;
	}
	
	public int getModel() {
		return mModel;
	}
	
	public int getTimeLimitMs() {
		return mTimeLimitMs;
	}
	
	public int getId() {
		return mId;
	}
	
	public int getProbaScaleupFactor() {
		return mProbaScaleUpFactor;
	}
}

//End of file
